"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = printHighlightedAst;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _generator = _interopRequireDefault(require("@babel/generator"));

var _parser = require("@babel/parser");

var _get = _interopRequireDefault(require("lodash/get"));

function getPathKeys(path) {
  var result = [];

  while (path) {
    result.push(path.key);
    if (path.parentKey && path.parentKey !== path.key) result.push(path.parentKey);
    path = path.parentPath;
  }

  return result.reverse();
}

function transformRange(code, start, end, transform) {
  return "".concat(code.substring(0, start)).concat(transform(code.substring(start, end))).concat(code.substring(end));
}

function printHighlightedAst(codeOrAst, _ref) {
  var highlights = _ref.highlights,
      generateOptions = _ref.generateOptions,
      parseOptions = _ref.parseOptions;
  var convertedHighlights = (0, _toConsumableArray2["default"])(highlights).sort(function (_ref2, _ref3) {
    var _ref4 = (0, _slicedToArray2["default"])(_ref2, 1),
        a = _ref4[0];

    var _ref5 = (0, _slicedToArray2["default"])(_ref3, 1),
        b = _ref5[0];

    var aIndex = a.node.start;
    var bIndex = b.node.start; // descending order

    return aIndex > bIndex ? -1 : aIndex < bIndex ? 1 : 0;
  }).map(function (_ref6) {
    var _ref7 = (0, _slicedToArray2["default"])(_ref6, 2),
        path = _ref7[0],
        highlighter = _ref7[1];

    return [getPathKeys(path), highlighter];
  });
  var printed = typeof codeOrAst === 'string' ? codeOrAst : (0, _generator["default"])(codeOrAst, generateOptions).code;
  var reparsed = (0, _parser.parse)(printed, parseOptions);
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = convertedHighlights[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _step$value = (0, _slicedToArray2["default"])(_step.value, 2),
          pathKeys = _step$value[0],
          highlight = _step$value[1];

      var node = (0, _get["default"])(reparsed, pathKeys);
      if (!node) continue;
      var start = node.start,
          end = node.end;
      if (start == null || end == null) continue;
      printed = transformRange(printed, start, end, highlight);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return printed;
}